@@ -1,7 +1,7 @@
 /*
  * aurora - Communications with Magnetek Aurora Inverter
  *
- * Copyright (C) 2006-2010 Curtis J. Blank curt@curtronics.com
+ * Copyright (C) 2006-2011 Curtis J. Blank curt@curtronics.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,10 +45,16 @@
  *				2. pass TransState the command description so if it gets a non-zero status it can
  *				   report for what command it got it
  *				3. added reporting for "Last four alarms"
+ * modified 30-jul-2011 cjb	1. fixed joule conversion error
+ * modified 06-aug-2011 cjb	1. fixed a integer conversion issue for -k due to some architectures
+ *                              2. fixed bizarreness with -bt
+ *                              3. adjust TimeBase to true UTC-0000 and take into account timezone in GetTime (-t)
+ *                                 SetTime (-S) functions
+ * modified 22-aug-2011 cjb	1. in SetTime check scanf return value rc
  *
  */
 
-char     VersionC[6] = "1.6.7";
+char     VersionC[7] = "1.7.0";
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -379,6 +385,80 @@ int GetMfgDate(int fdser, int yAddress)
     return(-1);
 }
 
+/* 
+
+*/
+int GetCEDaily(int fdser, int yAddress, int yGetEnergyDaily)
+{
+    int nCnt, lCnt = 1;
+    int addC = 0;
+    int loop = 1;
+    time_t tB, tVal;
+    struct tm tS;
+    char DT[9] = " ";
+    unsigned long days = 0, kwh, last_kwh = 0;
+
+    tS.tm_sec = tS.tm_min = tS.tm_hour = tS.tm_mon = tS.tm_isdst = 0;
+    tS.tm_mday = 1;
+    tS.tm_year = 100;
+    tB = mktime(&tS);
+    tS = *(localtime(&tB));
+    strftime(DT,9,"%Y%m%d ",&tS);
+
+    strcpy(szSerBuffer, _clearCMD);             /* clear Aurora cmd string */
+    szSerBuffer[cAddress] = yAddress;           /* set inverter address */
+    szSerBuffer[cCommand] = HIBYTE(opGetCEAdd); 
+    szSerBuffer[cParam1] = LOBYTE(opGetCEAdd);
+    szSerBuffer[cParam1End] = '\0';
+    nCnt = Communicate(fdser, yAddress);
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetCEAdd)) {
+        addC = cvrtShort(szSerBuffer);
+        if (bVerbose)
+            fprintf(stderr, "\nDaily Cumulated Energy Epoch %s Address 0x%04x\n",DT,(WORD)addC);
+        else
+            fprintf(outfp, "\n");
+        while (loop && nCnt > 0) {
+            strcpy(szSerBuffer, _clearCMD);
+            szSerBuffer[cAddress] = yAddress;           /* set inverter address */
+            szSerBuffer[cCommand] = opGetCEDaily;
+            szSerBuffer[cCEDailyAddH] = HIBYTE(addC);
+            szSerBuffer[cCEDailyAddL] = LOBYTE(addC);
+            szSerBuffer[cCEDailyEnd] = '\0';
+            nCnt = Communicate(fdser, yAddress);
+            if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetCEAdd)) {
+                days = cvrtShort(szSerBuffer);
+                kwh = cvrtShort(szSerBuffer+2);
+                if (kwh != 0xffff) {
+                    tVal = tB + ((days-1)*86400);
+                    tS = *(localtime(&tVal));
+                    strftime(DT,9,"%Y%m%d ",&tS);
+                    if (bVerbose)
+                        fprintf(stderr, "Daily Cumulated Energy Loop %3d Day %3d Address 0x%04x Days 0x%02x%02x Value 0x%02x%02x\td %li\t%s %7.3f kWh\n",loop,lCnt,(WORD)addC,(BYTE)szSerBuffer[aCEDailyDaysH],(BYTE)szSerBuffer[aCEDailyDaysL],(BYTE)szSerBuffer[aCEDailyValH],(BYTE)szSerBuffer[aCEDailyValL],days,DT,(float)kwh/1000);
+                    else
+                        fprintf(outfp, "%s %11.3f kWh\n",DT,(float)kwh/1000);
+                    addC = addC - 4;
+                    lCnt++;
+                } else {
+                    if (last_kwh == 0xffff) {
+                        fprintf(stderr, "\nDaily Cumulated Energy retreival problem, may not have %d days recorded\n",yGetEnergyDaily);
+                        return(0);
+                    }
+                    if (bVerbose) fprintf(stderr, "Daily Cumulated Energy Loop %3d Loop to memory block beginning\n",loop);
+                    addC = addC + (366*4);
+                }
+                last_kwh = kwh;
+                loop++;
+            } else 
+                loop = 0;
+            if (loop > 367 || lCnt > yGetEnergyDaily) loop = 0;
+        }
+        return(0);
+    } else if (bVerbose)
+        fprintf(stderr, "\nDaily Cumulated Energy Address retreival problem\n");
+
+    return(-1);
+}
+
 
 /*--------------------------------------------------------------------------
     GetCE
@@ -404,25 +484,25 @@ int GetCE(int fdser, int yAddress)
     }
     else
     {
-        fprintf(outfp, "\n%-26s = %11.3f KWh",_CEpar0,DAILY/1000.0);
+        fprintf(outfp, "\n%-26s = %11.3f kWh",_CEpar0,DAILY/1000.0);
         if(yCost > 0) fprintf(outfp, "\t($%8.3f)",(DAILY/1000.0)*yCost);
 
-        fprintf(outfp, "\n%-26s = %11.3f KWh",_CEpar1,WEEKLY/1000.0);
+        fprintf(outfp, "\n%-26s = %11.3f kWh",_CEpar1,WEEKLY/1000.0);
         if(yCost > 0) fprintf(outfp, "\t($%8.3f)",(WEEKLY/1000.0)*yCost);
 
-//        fprintf(outfp, "\n%-26s = %11.3f KWh",_CEpar2,LAST7DAYS/1000.0);	/* see above note */
+//        fprintf(outfp, "\n%-26s = %11.3f kWh",_CEpar2,LAST7DAYS/1000.0);	/* see above note */
 //        if(yCost > 0) fprintf(outfp, "\t($%8.3f)",(LAST7DAYS/1000.0)*yCost);
 
-        fprintf(outfp, "\n%-26s = %11.3f KWh",_CEpar3,MONTHLY/1000.0);
+        fprintf(outfp, "\n%-26s = %11.3f kWh",_CEpar3,MONTHLY/1000.0);
         if(yCost > 0) fprintf(outfp, "\t($%8.3f)",(MONTHLY/1000.0)*yCost);
 
-        fprintf(outfp, "\n%-26s = %11.3f KWh",_CEpar4,YEARLY/1000.0);
+        fprintf(outfp, "\n%-26s = %11.3f kWh",_CEpar4,YEARLY/1000.0);
         if(yCost > 0) fprintf(outfp, "\t($%8.3f)",(YEARLY/1000.0)*yCost);
 
-        fprintf(outfp, "\n%-26s = %11.3f KWh",_CEpar5,TOTAL/1000.0);
+        fprintf(outfp, "\n%-26s = %11.3f kWh",_CEpar5,TOTAL/1000.0);
         if(yCost > 0) fprintf(outfp, "\t($%8.3f)",(TOTAL/1000.0)*yCost);
 
-        fprintf(outfp, "\n%-26s = %11.3f KWh",_CEpar6,PARTIAL/1000.0);
+        fprintf(outfp, "\n%-26s = %11.3f kWh",_CEpar6,PARTIAL/1000.0);
         if(yCost > 0) fprintf(outfp, "\t($%8.3f)",(PARTIAL/1000.0)*yCost);
 
         fprintf(outfp, "\n");
@@ -698,7 +778,7 @@ int GetJoules(int fdser, int yAddress)
     if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetEnergy10Sec)) {
         Joules = cvrtShort(szSerBuffer);
         if(bVerbose) fprintf(stderr, "Joules       %12lu\n",(unsigned long)Joules);
-        fprintf(outfp, "\nEnergy in the last 10 seconds (Joules) : %11.6f\n",(unsigned long)Joules*0.319509);
+        fprintf(outfp, "\nEnergy in the last 10 seconds (Joules) : %lu\n",(unsigned long)Joules);
         return(0);
    }
     return(-1);
@@ -712,10 +792,11 @@ int GetTime(int fdser, int yAddress)
 {
     int nCnt = 0;
     time_t timeValLong;
+    long int gmtoff = 0;
+    long int gmtoffTB = 0;
     struct tm tim;
     char curTime[24];
     char fromInv[10] = "Inverter \0";
-    BOOL DLS = TRUE;
 
     if(bGetInvTime) {
         strcpy(szSerBuffer, _clearCMD);		/* clear Aurora cmd string */
@@ -726,15 +807,27 @@ int GetTime(int fdser, int yAddress)
     }
     if (nCnt > 0) {
         if (! TransState((int)szSerBuffer[aState],_opGetTime)) return(-1);
+        timeValLong = time(NULL);
+        tim = *(localtime(&timeValLong));
+        gmtoff = tim.tm_gmtoff;
+        timeValLong = (time_t)TimeBase;
+        tim = *(localtime(&timeValLong));
+        gmtoffTB = tim.tm_gmtoff;
         if(bVerbose) {
-            timeValLong = (time_t)TimeBase;
+            timeValLong = (time_t)TimeBase - gmtoffTB;
             fprintf(stderr, "\nTimeBase     %12lu\n",(time_t)TimeBase);
-            fprintf(stderr, "Base Inverter date/time: %s\n",ctime(&timeValLong));
+            fprintf(stderr, "gmtoffTB     %12li\n",gmtoffTB);
+            tim = *(localtime(&timeValLong));
+            strftime(curTime,24,"%d-%b-%Y %H:%M:%S\n",&tim);
+            fprintf(stderr, "Base Inverter date/time: %s\n",curTime);
         }
         timeValLong = (time_t)cvrtLong(szSerBuffer);
-        if(bVerbose) fprintf(stderr, "timeValLong  %12lu\n",(time_t)timeValLong);
+        if(bVerbose) {
+            fprintf(stderr, "timeValLong  %12lu\n",(time_t)timeValLong);
+            fprintf(stderr, "gmtoff       %12li\n",gmtoff);
+        }
         timeValLong += (time_t)TimeBase;
-        if(DLS) timeValLong -= (time_t)(60*60);		/* adjust for daylight savings time */
+        timeValLong -= gmtoff;
         if(bVerbose) fprintf(stderr, "timeValLong  %12lu\n",(time_t)timeValLong);
     }
     if(bGetLocTime) {
@@ -767,34 +860,32 @@ int SetTime(int fdser, int yAddress)
     int rc;
     time_t timeValLong;
     struct tm tim;
+    long int gmtoff = 0;
     char curTime[24];
     char answer;
 
     printf("\n**** WARNING *****   ***** WARNING *****   ***** WARNING ****\n");
     printf("Setting the Date and Time has been known to clear all History\n");
+    printf("   Except \"Total Energy\" but no guarantees that it won't\n");
     printf("               (Not enabled on all models.)\n");
     printf("\nAre you sure your want to proceed? y/[n] : ");
     rc = scanf("%c", &answer);
-    if(answer != 'y' && answer != 'Y') return(1);
+    if(rc == EOF || (answer != 'y' && answer != 'Y')) return(1);
 
-    timeValLong = time((time_t *) NULL);
+    timeValLong = time(NULL);
+    tim = *(localtime(&timeValLong));
+    gmtoff = tim.tm_gmtoff;
     if(bVerbose) {
 	fprintf(stderr, "\ntimeValLong        %12lu\n",(time_t)timeValLong);
-        tim = *(localtime(&timeValLong));
         strftime(curTime,24,"%d-%b-%Y %H:%M:%S",&tim);
         fprintf(stderr, "setting time to %s DST %i\n",curTime,tim.tm_isdst);
     }
 
     /* adjust time to Aurora's time base */
     timeValLong -= (long)TimeBase;
+    timeValLong += gmtoff;
     if(bVerbose) fprintf(stderr, "timeValLong        %12lu\n",(time_t)timeValLong);
 
-    /* adjust for Daylight Saving Time */
-    if (tim.tm_isdst == 1) {
-        timeValLong += 3600;
-        if(bVerbose) fprintf(stderr, "timeValLong (DST)  %12lu\n",(time_t)timeValLong);
-    }
-
     strcpy(szSerBuffer, _clearCMD);		/* clear Aurora cmd string */
     szSerBuffer[cAddress] = yAddress;		/* set inverter address */
     szSerBuffer[cCommand] = opSetTime;
@@ -896,7 +987,7 @@ int Communicate(int fdser, int yAddress)
     long long int curtvusecs = 0, lastcommtvusecs = 0, elapsedtvusecs = 0;
 
     strcpy(szSerBufferSave, _clearCMD);
-    strcpy(szSerBufferSave, szSerBuffer);
+    memcpy(szSerBufferSave, szSerBuffer,_szSerBufferLen);
     gettimeofday(&curtv, NULL);
     while(CRCrc < 0 && attempts <= yMaxAttempts) {
         if(lastcommtv.tv_sec == 0 || lastcommtv.tv_usec == 0) {
@@ -907,10 +998,10 @@ int Communicate(int fdser, int yAddress)
             lastcommtvusecs = (lastcommtv.tv_sec*1000000) + lastcommtv.tv_usec;
             elapsedtvusecs = curtvusecs - lastcommtvusecs;
         }
-        strcpy(szSerBuffer, szSerBufferSave);
-        if(bVerbose) fprintf(stderr, "\nElapsed time since last comm %llu uS\nAttempt %i",elapsedtvusecs,attempts);
+        memcpy(szSerBuffer, szSerBufferSave,_szSerBufferLen);
+        if(bVerbose) fprintf(stderr, "\nElapsed time since last comm %llu us\nAttempt %i",elapsedtvusecs,attempts);
         if(yCommPause > 0 && elapsedtvusecs < yCommPause) {
-            if(bVerbose) fprintf(stderr, " Sleeping for %llu uS",yCommPause-elapsedtvusecs);
+            if(bVerbose) fprintf(stderr, " Sleeping for %llu us",yCommPause-elapsedtvusecs);
             usleep(yCommPause-elapsedtvusecs);
         }
         if(bVerbose) fprintf(stderr, "\nClearing read buffer  ");
@@ -1029,7 +1120,7 @@ float *cvrtFloat(char *Buffer)
     cValue[3] = Buffer[aParam1];
 
     value = (float *)cValue;
-    if(bVerbose) fprintf(stderr, "cvrtFloat %12.6f\n",*value);
+    if(bVerbose) fprintf(stderr, "cvrtFloat %12.6f 0x%02x%02x%02x%02x\n",*value,cValue[3],cValue[2],cValue[1],cValue[0]);
 
     return(value);
 }
@@ -1049,7 +1140,7 @@ unsigned long cvrtLong(char *Buffer)
     cValue[3] = Buffer[aParam1] & 0xff;
 
     value = (unsigned long *)cValue;
-    if(bVerbose) fprintf(stderr, "cvrtLong     %12lu\n",*value);
+    if(bVerbose) fprintf(stderr, "cvrtLong     %12lu 0x%02x%02x%02x%02x\n",*value,cValue[3],cValue[2],cValue[1],cValue[0]);
 
     return(*value);
 }
@@ -1066,9 +1157,11 @@ unsigned long cvrtShort(char *Buffer)
 
     cValue[0] = Buffer[aParam2] & 0xff;
     cValue[1] = Buffer[aParam1] & 0xff;
+    cValue[2] = 0x00;
+    cValue[3] = 0x00;
 
     value = (unsigned long *)cValue;
-    if(bVerbose) fprintf(stderr, "cvrtShort    %12lu\n",*value);
+    if(bVerbose) fprintf(stderr, "cvrtShort    %12lu 0x%02x%02x%02x%02x\n",*value,cValue[3],cValue[2],cValue[1],cValue[0]);
 
     return(*value);
 }
@@ -1130,7 +1223,7 @@ int ReadNextChar(int nfd, char *pChar, int timeout)
         exit(2);
     }
     if(bVerbose) fprintf(stderr, "RC=%i (%02x)",nResult,(unsigned char)*pChar);
-    if(bVerbose && timeout > 0 && (cUsecs-sUsecs) > 0) fprintf(stderr, " Read waited %i uS.",(int)(cUsecs-sUsecs));
+    if(bVerbose && timeout > 0 && (cUsecs-sUsecs) > 0) fprintf(stderr, " Read waited %i us.",(int)(cUsecs-sUsecs));
     return nResult;
 
 }
@@ -1150,7 +1243,7 @@ int ReadToBuffer(int nfd, char *pszBuffer, int nBufSize)
     int sanity = 0;
 
     while(attempts < nBufSize && sanity < 100) {
-        if(bVerbose) fprintf(stderr, "read attempts %i timeout %iuS ",attempts+1,yTimeout);
+        if(bVerbose) fprintf(stderr, "read attempts %i timeout %ius ",attempts+1,yTimeout);
         rc = ReadNextChar(nfd, pBuf++, yTimeout);
         if(rc < 0)
             return nPos;	/* no character available */
