@@ -1,7 +1,7 @@
 /*
  * aurora - Communications with Magnetek Aurora Inverter
  *
- * Copyright (C) 2006-2010 Curtis J. Blank curt@curtronics.com
+ * Copyright (C) 2006-2011 Curtis J. Blank curt@curtronics.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
  *
  */
 
-#define	VersionNH	"1.6.7"
+#define	VersionNH	"1.6.8"
 
 /* # 50 State Request */
 
@@ -245,6 +245,13 @@
 #define aRel1   4
 #define aRel0   5
 
+/* # 75 Energy cumulated each day address ** Experimental ** */
+
+#define opGetCEAdd	0x4b02	/* 75 02 */
+#define _opGetCEAdd "Daily Cumulated Energy Address"
+
+#define aCEAddH		2
+#define aCEAddL		3
 
 /* # 76 Energy cumulated in the last 10 seconds */
 
@@ -292,6 +299,20 @@
 #define _CEpar5  "Total Energy"
 #define _CEpar6  "Partial Energy"
 
+/* # 79 Daily Cumulated Energy ** Experimental ** */
+
+#define opGetCEDaily	79
+#define _opGetCEDaily "Daily Cumulated Energy"
+
+#define cCEDailyAddH	2
+#define cCEDailyAddL	3
+#define cCEDailyEnd	4
+
+#define aCEDailyDaysH	2
+#define aCEDailyDaysL	3
+#define aCEDailyValH	4
+#define aCEDailyValL	5
+
 /* # 80 Time Counter */
 
 #define	opGetCounters	80
