@@ -1,7 +1,7 @@
 /*
  * aurora - Communications with Magnetek Aurora Inverter
  *
- * Copyright (C) 2006-2010 Curtis J. Blank curt@curtronics.com
+ * Copyright (C) 2006-2011 Curtis J. Blank curt@curtronics.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,11 +45,16 @@
  * modified 28-mar-2010 cjb	1. working on adding more DSP information
  * modified 27-jul-2010 cjb	1. added -P option to throttle commands sent to the inverter
  * modified 21-sep-2010 cjb	1. added -A option to read "Last four alarms"
+ * modified 22-jan-2011 cjb	1. added -k option to get up to a years worth of daily generation history
+ * modified 19-mar-2011 cjb	1. added -X option to enable RTS/CTS on the serial port
+ * modified 13-aug-2011 cjb	1. increase the allowed values from 31 to 63 for inverter address (-a)
+ *                                 as allowed in newer inverters
+ * modified 22-aug-2011 cjb	1. clean up compile 'unused-but-set' messages
  *
  */
 
-static char	VersionM[] = "1.6.7";
-char     VersionC[6];
+static char	VersionM[] = "1.7.0";
+char     VersionC[7];
 
 #include <syscall.h>
 #include <unistd.h>
@@ -85,9 +90,10 @@ BOOL bCommCheck = FALSE;
 BOOL bColOutput = FALSE;
 BOOL bCalcGridPwr = FALSE;
 BOOL bXonXoff = FALSE;
+BOOL bRTSCTS = FALSE;
 BOOL bRptReties = FALSE;
 BOOL bRptReadPause = FALSE;
-int yTimeout = 0;			/* read timeout value in uS */
+int yTimeout = 0;			/* read timeout value in us */
 long PID;
 int yMaxAttempts = 1;
 int yReadPause = 0;
@@ -106,7 +112,7 @@ float yCost = 0.0;
 static char VersionCHc[6] = VersionCH;
 static char VersionMHc[6] = VersionMH;
 static char VersionNHc[6] = VersionNH;
-static int yAddress = 0;			/* Inverter address 1-31 */
+static int yAddress = 0;			/* Inverter address 1-63 */
 static char szttyDevice[255];			/* Serial device string */
 static char outfile[512];			/* output file FQN */
 static BOOL bHelp = FALSE;			/* Print the help text */
@@ -124,6 +130,7 @@ static BOOL bVersion = FALSE;			/* Display program version */
 static BOOL bGetTime;
 static BOOL bGetLastAlarms = FALSE;		/* Get last four alarms */
 static BOOL bFileError = FALSE;
+static int yGetEnergyDaily = 0;
 static unsigned char yDelay;			/* Read wait time */
 static int yLockWait = 0;			/* Seconds to wait to lock serial port */
 static char devLCKfile[255] = ttyLCKloc;
@@ -147,6 +154,7 @@ int main(int argc, char *argv[])
     struct tm timStruct;
     long rPID;
     int bRead, bWrite, lckCNT;
+    int errno_save = 0;
     char cmdFile[255];
     char command[255];
     char *SubStrPos = NULL;
@@ -166,10 +174,10 @@ int main(int argc, char *argv[])
         printf("Options:\n");
         printf(" -A, --last-alarms              Get last four alarms (once displayed FIFO queue is cleared)\n");
         printf(" -b, --verbose                  Verbose mode\n");
-        printf(" -C, --calc-value=num           Calculate $ value using num * KWh\n");
-        printf(" -c, --columnize                Output data in columns --> for -d, -e, -D, & -E options only\n");
-        printf("                                  will disable all other options -- if value ends with an \"*\"\n");
-        printf("                                  reporting of that item may not be in inverters firmware\n");
+        printf(" -C, --calc-value=<num>         Calculate $ value using <num> * kWh\n");
+        printf(" -c, --columnize                Output data in columns --> for -d, -e, -D, -E, -t, & -T options\n");
+        printf("                                  only will disable any other options -- if value ends with an\n");
+        printf("                                  \"*\" reporting of that item may not be in inverters firmware\n");
         printf(" -d, --get-dsp                  Get DSP data\n");
         printf(" -D, --get-dsp-extended         Get more DSP data\n");
         printf(" -e, --get-energy               Get Cumulated Energy readings\n");
@@ -179,13 +187,17 @@ int main(int argc, char *argv[])
         printf(" -h, --help                     This text\n");
         printf(" -i, --get-count                Display Inverter Time Counters\n");
         printf(" -j, --get-joules               Display Energy cumulated in the last 10 seconds\n");
-        printf(" -l, --delay=num                Cmd Delay in 1/10ths seconds. Default is 1 (0.1 sec)\n");
+        printf(" -k, --daily-kwh=<num>          Get past daily kWh for <num> days (1-366) ** Experimental **\n");
+        printf("                                  <num> is optional for long option and if omitted all\n");
+        printf("                                  366 days or as many that are found will be displayed\n");
+        printf(" -l, --delay=<num>              Serial port timeout in for read. <num> in 1/10ths seconds\n");
+        printf("                                  Default is 1 (0.1 sec)\n");
         printf(" -m, --get-conf                 Query for Inverter System Configuration\n");
         printf(" -n, --serial-number            Query for Inverter Serial Number\n");
         printf(" -o, --output-file=<filename>   Append data to file (Created if nonexistant)\n");
         printf(" -p, --part-number              Query for Inverter Part Number\n");
-        printf(" -P, --comm-pause=num           Wait <num> uS between sending commands to inverter (1-1000000)\n");
-	printf(" -R, --read-timeout=num		Timeout value when reading data from the Inverter (mS)\n");
+        printf(" -P, --comm-pause=<num>         Wait <num> us between sending commands to inverter (1-1000000)\n");
+	printf(" -R, --read-timeout=<num>       Timeout value when reading data from the Inverter (ms)\n");
         printf(" -r, --calc-grid-power          Calc Grid power using Grid Voltage * Grid Current,\n");
         printf("                                  instead of reporting the Inverter's value. --> for \n");
         printf("                                  -d option only, ignored when used with -c option\n");
@@ -197,15 +209,17 @@ int main(int argc, char *argv[])
         printf(" -t, --get-invtime              Display Inverter Date/Time\n");
         printf(" -V, --version                  Aurora communications program version\n");
         printf(" -v, --inv-version              Query for Version string\n");
-	printf(" -U, --read-pause=num           Pause 'num' milli-seconds after sending command to inverter before\n");
+	printf(" -U, --read-pause=<num>         Pause <num> milli-seconds after sending command to inverter before\n");
         printf("                                  reading response from inverter (1-10000)\n");
         printf(" -u, --rpt-read-pause           Report when/that pausing before read\n");
         printf(" -w, --lock-wait                Seconds to wait to lock serial port. (1-30)\n");
-	printf(" -x, --xon-xoff			Enable XON/XOFF on the serial port.\n");
-        printf(" -Y, --retries=num		Retry failed communications with inverter up to 'num' times (1-100)\n");
-        printf(" -y, --rpt-retries		Report the number of retires done\n");
-        printf("\n -a, --address=num              Inverter address. (1-31) *** Required parameter ***\n");
-        printf(" Device                         Serial Device.           *** Required parameter ***\n");
+        printf(" -X, --rts-cts                  Enable RTS/CTS on the serial port.\n");
+	printf(" -x, --xon-xoff                 Enable XON/XOFF on the serial port.\n");
+        printf(" -Y, --retries=<num>            Retry failed communications with inverter up to <num> times (1-100)\n");
+        printf(" -y, --rpt-retries              Report the number of retires done\n\n");
+        printf("               *** Required Parameters ***\n");
+        printf(" -a, --address=<num>            Inverter address. 1-31 on older inverters, 1-63 on newer inverters.\n");
+        printf(" Device                         Serial Device.\n");
         printf("\n");
         if (bHelp) 
             exit(0);
@@ -213,7 +227,7 @@ int main(int argc, char *argv[])
             exit(2);
     }
 
-    if(bVerbose) fprintf(stderr, "\nRunTime %s\n",RunTime);
+    if(bVerbose) fprintf(stderr, "\nRunTime %s v%-6s\n",RunTime,VersionM);
 
     if(bVerbose) fprintf (stderr, "PID : %ld\n", PID);
 
@@ -230,7 +244,13 @@ int main(int argc, char *argv[])
         exit(2);
     }
     bWrite = fprintf(fdserlck, "%ld\n", PID);
+    errno_save = errno;
     fclose(fdserlck);
+    if (bWrite < 0 || errno_save != 0) {
+        if(bVerbose) fprintf(stderr, "\n");
+        fprintf(stderr, "%s: %s: Problem locking serial device, can't write lock file: %s.\n%s\n\n",RunTime,ProgramName,devLCKfile,strerror (errno_save));
+        exit(2);
+    }
 
     rPID = 0;
     lckCNT = -1;
@@ -242,7 +262,14 @@ int main(int argc, char *argv[])
             exit(2);
         }
         bRead = fscanf(fdserlck, "%ld", &rPID);
+        errno_save = errno;
         fclose(fdserlck);
+        if(bRead == EOF || errno_save != 0) {
+            if(bVerbose) fprintf(stderr, "\n");
+            fprintf(stderr, "%s: %s: Problem locking serial device, can't read lock file: %s.\n%s\n\n",RunTime,ProgramName,devLCKfile,strerror (errno_save));
+            exit(2);
+        }
+
         sprintf(cmdFile,"/proc/%ld/cmdline",rPID);
         fdserlck = fopen(cmdFile, "r");
         if (fdserlck != NULL) {
@@ -283,11 +310,14 @@ int main(int argc, char *argv[])
 
     memset(&newtio, 0, sizeof(newtio));
 
-    newtio.c_cflag &= ~CRTSCTS;			/* disable hardware flow control */
     newtio.c_cflag &= ~PARENB;			/* no parity */
     newtio.c_cflag &= ~CSTOPB;			/* on stop bit */
     newtio.c_cflag &= ~CSIZE;			/* character size mask */
     newtio.c_cflag &= ~HUPCL;			/* no hangup */
+    if (bRTSCTS)
+        newtio.c_cflag |= ~CRTSCTS;             /* enable hardware flow control */
+    else
+        newtio.c_cflag &= ~CRTSCTS;             /* disable hardware flow control */
     newtio.c_cflag |= CS8 | CLOCAL | CREAD;	/* 8 bit - ignore modem control lines - enable receiver */
     if (bXonXoff)
         newtio.c_iflag |= IXON | IXOFF;		/* enable XON/XOFF flow control on output & input */
@@ -432,6 +462,10 @@ int main(int argc, char *argv[])
 
             if (rc == 0 && bGetLastAlarms)
                 rc |= GetLastAlarms(fdser,yAddress);
+
+            if (rc == 0 && yGetEnergyDaily > 0)
+                rc |= GetCEDaily(fdser,yAddress,yGetEnergyDaily);
+
         }
 
         if (rc == 0 && bGetDSP)
@@ -526,6 +560,7 @@ int ClrSerLock(long PID) {
     FILE *fdserlck, *fdserlcknew;
     long rPID;
     int bWrite, bRead;
+    int errno_save = 0;
 
     if(bVerbose) fprintf(stderr, "\ndevLCKfile <%s>\ndevLCKfileNew <%s>\nClearing Serial Port Lock (%ld)...", devLCKfile, devLCKfileNew, PID);
     fdserlck = fopen(devLCKfile, "r");
@@ -543,7 +578,13 @@ int ClrSerLock(long PID) {
     }
     bRead = fscanf(fdserlck, "%ld", &rPID);
     while (bRead != EOF) {
-        if (rPID != PID) bWrite = fprintf(fdserlcknew, "%ld\n", rPID);
+        if (rPID != PID) {
+            bWrite = fprintf(fdserlcknew, "%ld\n", rPID);
+            errno_save = errno;
+            if (bWrite < 0 || errno_save != 0) {
+                fprintf(stderr, "%s: %s: Problem clearing serial device lock, can't write lock file: %s.\n%s\n\n",RunTime,ProgramName,devLCKfile,strerror (errno_save));
+            }
+        }
         bRead = fscanf(fdserlck, "%ld", &rPID);
     }
     fclose(fdserlck);
@@ -567,7 +608,13 @@ int GetParms(int argc, char *argv[])
     BOOL b = FALSE;
     char *pos;
 
-    if (strchr(VersionM,'x') != NULL) fprintf(stderr, "\n**** THIS IS EXPERIMENTAL CODE ****\n"); 
+    if (strpbrk(VersionM,"abcdefghijklmnopqurtsuvwxyz") != NULL) fprintf(stderr, "\n**** THIS IS EXPERIMENTAL CODE ****\n");
+    if (strcmp(VersionM,VersionC) != 0) {
+        fprintf(stderr, "\n**** MODULE VERSION MISMATCH ****\n");
+        fprintf(stderr, "      Main module : %-6s\n",VersionM);
+        fprintf(stderr, "      Comm module : %-6s\n",VersionC);
+        return 0;
+    }
 
     while (! b && i < argc) if (strcmp(argv[i++],"-b") == 0) b = TRUE;
 
@@ -576,7 +623,7 @@ int GetParms(int argc, char *argv[])
         i = 0;
         while (i < argc) fprintf(stderr, "%s ",argv[i++]);
         fprintf(stderr, "\n");
-   }
+    }
 
      /* options descriptor */
     static struct option longopts[] = {
@@ -595,10 +642,11 @@ int GetParms(int argc, char *argv[])
         { "help",		no_argument,		0,	'h' },
         { "get-count",		no_argument,		0,	'i' },
         { "get-joules",		no_argument,		0,	'j' },
+        { "daily-kwh",          optional_argument,      0,      'k' },
 	{ "delay",		required_argument,	0,	'l' },
-	{ "output-file",	required_argument,	0,	'o' },
         { "get-conf",		no_argument,		0,      'm' },
         { "serial-number",	no_argument,		0,	'n' },
+        { "output-file",        required_argument,      0,      'o' },
         { "part-number",	no_argument,		0,	'p' },
         { "comm-pause",		required_argument,	0,	'P' },
         { "read-timeout",	required_argument,	0,	'R' },
@@ -612,6 +660,7 @@ int GetParms(int argc, char *argv[])
 	{ "read-pause",		required_argument,	0,	'U' },
 	{ "rpt-read-pause",	no_argument,		0,	'u' },
         { "lock-wait",		required_argument,	0,	'w' },
+        { "rts-cts",		no_argument,		0,	'X' },
 	{ "xon-xoff",		no_argument,		0,	'x' },
 	{ "retries",		required_argument,	0,	'Y' },
 	{ "rpt-retries",	no_argument,		0,	'y' },
@@ -626,7 +675,7 @@ int GetParms(int argc, char *argv[])
     if(argc == 1)
         return 0;				/* no parms at all */
 
-    while ((c = getopt_long(argc, argv, "a:AbC:cDdEefgHhijl:mno:P:pR:rSsTtVvU:uw:xY:y", longopts, NULL )) != EOF) {
+    while ((c = getopt_long(argc, argv, "a:AbC:cDdEefgHhijk:l:mno:P:pR:rSsTtVvU:uw:XxY:y", longopts, NULL )) != EOF) {
         switch (c) {
             case 'a':
                 /* Inverter address */
@@ -648,11 +697,24 @@ int GetParms(int argc, char *argv[])
             case 'h': bHelp        = TRUE; break;
             case 'i': bGetCount    = TRUE; break;
             case 'j': bGetJoules   = TRUE; break;
+            case 'k':
+                /* Get ECC */
+                if (optarg == NULL) {
+                    yGetEnergyDaily = 366;
+                } else {
+                    i = atoi(optarg);
+                    if(i < 1 || i > 366) {
+                        fprintf(stderr, "\n%s: %s: -k value out of range, 1 to 366.\n",RunTime,ProgramName);
+                        return 0;
+                    }
+                    yGetEnergyDaily = i;
+                }
+                break;
             case 'l':
                 /* Get delay time */
                 i = atoi(optarg);
                 if(i < 0 || i > 255) {
-                        fprintf(stderr, "\n%s: %s: Illegal delay specified.\n",RunTime,ProgramName);
+                        fprintf(stderr, "\n%s: %s: -l Illegal delay specified.\n",RunTime,ProgramName);
                         return 0;
                 }
                 yDelay = (unsigned char)i;
@@ -664,12 +726,12 @@ int GetParms(int argc, char *argv[])
             case 'P':
                 yCommPause = atoi(optarg);
                 if (yCommPause <= 0 || yCommPause > 1000000) {
-                    fprintf(stderr, "\n%s: %s: Comm Pause micro-seconds (%li) out of range, 1-1000000.\n",RunTime,ProgramName,yCommPause);
+                    fprintf(stderr, "\n%s: %s: -P Comm Pause micro-seconds (%li) out of range, 1-1000000.\n",RunTime,ProgramName,yCommPause);
                     return 0;
                 }
                 break;
             case 'R':
-                /* read timeout value in uS */
+                /* read timeout value in us */
                 yTimeout = atoi(optarg)*1000;
                 break;
             case 'r': bCalcGridPwr = TRUE; break;
@@ -680,7 +742,7 @@ int GetParms(int argc, char *argv[])
             case 'U':
                 yReadPause = atoi(optarg);
                 if (yReadPause < 1 || yReadPause > 10000) {
-                    fprintf(stderr, "\n%s: %s: Read Pause milli-seconds (%d) out of range, 1-10000.\n",RunTime,ProgramName,yReadPause);
+                    fprintf(stderr, "\n%s: %s: -U Read Pause milli-seconds (%d) out of range, 1-10000.\n",RunTime,ProgramName,yReadPause);
                     return 0;
                 }
                 break;
@@ -688,15 +750,16 @@ int GetParms(int argc, char *argv[])
             case 'w':
                 yLockWait = atoi(optarg);
                 if (yLockWait < 1 || yLockWait > 30) {
-                    fprintf(stderr, "\n%s: %s: Lock Wait seconds (%d) out of range, 1-30.\n",RunTime,ProgramName,yLockWait);
+                    fprintf(stderr, "\n%s: %s: -w Lock Wait seconds (%d) out of range, 1-30.\n",RunTime,ProgramName,yLockWait);
                     return 0;
                 }
                 break;
+            case 'X': bRTSCTS      = TRUE; break;
             case 'x': bXonXoff     = TRUE; break;
             case 'Y':
                 yMaxAttempts = atoi(optarg);
                 if (yMaxAttempts < 1 || yMaxAttempts > 100) {
-                    fprintf(stderr, "\n%s: %s: Retries (%d) out of range, 1-100.\n",RunTime,ProgramName,yMaxAttempts);
+                    fprintf(stderr, "\n%s: %s: -Y Retries (%d) out of range, 1-100.\n",RunTime,ProgramName,yMaxAttempts);
                     return 0;
                 }
                 break;
@@ -726,9 +789,9 @@ int GetParms(int argc, char *argv[])
     } else {
         *devLCKfile = '\0';
     }
-    if(bVerbose) fprintf(stderr, "\nszttyDevice: %s\nyDelay:  %i\nyTimeout %i uS\ndevLCKfile: <%s>\ndevLCKfileNew: <%s>\n",szttyDevice,yDelay,yTimeout,devLCKfile,devLCKfileNew);
+    if(bVerbose) fprintf(stderr, "\nszttyDevice: %s\nyDelay:  %i\nyTimeout %i us\ndevLCKfile: <%s>\ndevLCKfileNew: <%s>\n",szttyDevice,yDelay,yTimeout,devLCKfile,devLCKfileNew);
 
-    if (yAddress < 1 || yAddress > 31) {
+    if (yAddress < 1 || yAddress > 63) {
         fprintf(stderr, "\n%s: %s: Illegal address (%d) specified.\n",RunTime,ProgramName,yAddress);
         return 0;
     }
@@ -742,11 +805,11 @@ int GetParms(int argc, char *argv[])
 void Version()
 {
     printf("\nAurora module versions:\n");
-    printf("Main module : %6s\n",VersionM);
-    printf("Comm module : %6s\n",VersionC);
-    printf("main.h      : %6s\n",VersionMHc);
-    printf("comm.h      : %6s\n",VersionCHc);
-    printf("names.h     : %6s\n",VersionNHc);
-    printf("states.h    : %6s\n\n",VersionSHc);
+    printf("Main module : %-6s\n",VersionM);
+    printf("Comm module : %-6s\n",VersionC);
+    printf("main.h      : %-6s\n",VersionMHc);
+    printf("comm.h      : %-6s\n",VersionCHc);
+    printf("names.h     : %-6s\n",VersionNHc);
+    printf("states.h    : %-6s\n\n",VersionSHc);
 }
  
